@@ -1,13 +1,11 @@
 package HTML::Form;
 
-# $Id: Form.pm,v 1.51 2005/12/06 09:17:35 gisle Exp $
-
 use strict;
 use URI;
 use Carp ();
 
 use vars qw($VERSION);
-$VERSION = sprintf("%d.%03d", q$Revision: 1.51 $ =~ /(\d+)\.(\d+)/);
+$VERSION = "5.810";
 
 my %form_tags = map {$_ => 1} qw(input textarea button select option);
 
@@ -17,13 +15,13 @@ my %type2class = (
  hidden   => "TextInput",
  textarea => "TextInput",
 
- button   => "IgnoreInput",
  "reset"  => "IgnoreInput",
 
  radio    => "ListInput",
  checkbox => "ListInput",
  option   => "ListInput",
 
+ button   => "SubmitInput",
  submit   => "SubmitInput",
  image    => "ImageInput",
  file     => "FileInput",
@@ -58,16 +56,18 @@ The following methods are available:
 
 =over 4
 
-=item @forms = HTML::Form->parse( $html_document, $base_uri )
-
 =item @forms = HTML::Form->parse( $response )
 
+=item @forms = HTML::Form->parse( $html_document, $base )
+
+=item @forms = HTML::Form->parse( $html_document, %opt )
+
 The parse() class method will parse an HTML document and build up
 C<HTML::Form> objects for each <form> element found.  If called in scalar
 context only returns the first <form>.  Returns an empty list if there
 are no forms to be found.
 
-The $base_uri is the URI used to retrieve the $html_document.  It is
+The $base is the URI used to retrieve the $html_document.  It is
 needed to resolve relative action URIs.  If the document was retrieved
 with LWP then this this parameter is obtained from the
 $response->base() method, as shown by the following example:
@@ -87,18 +87,44 @@ directly, so the example above can be more conveniently written as:
 Note that any object that implements a decoded_content() and base() method
 with similar behaviour as C<HTTP::Response> will do.
 
+Finally options might be passed in to control how the parse method
+behaves.  The following options are currently recognized:
+
+=over
+
+=item C<base>
+
+Another way to provide the base URI.
+
+=item C<verbose>
+
+Print messages to STDERR about any bad HTML form constructs found.
+
+=back
+
 =cut
 
 sub parse
 {
-    my($class, $html, $base_uri) = @_;
+    my $class = shift;
+    my $html = shift;
+    unshift(@_, "base") if @_ == 1;
+    my %opt = @_;
+
     require HTML::TokeParser;
     my $p = HTML::TokeParser->new(ref($html) ? $html->decoded_content(ref => 1) : \$html);
     eval {
 	# optimization
-	$p->report_tags(qw(form input textarea select optgroup option keygen label));
+	$p->report_tags(qw(form input textarea select optgroup option keygen label button));
     };
 
+    my $base_uri = delete $opt{base};
+    my $verbose = delete $opt{verbose};
+
+    if ($^W) {
+	Carp::carp("Unrecognized option $_ in HTML::Form->parse") for sort keys %opt;
+    }
+
     unless (defined $base_uri) {
 	if (ref($html)) {
 	    $base_uri = $html->base;
@@ -155,6 +181,10 @@ sub parse
 		    my $type = delete $attr->{type} || "text";
 		    $f->push_input($type, $attr);
 		}
+                elsif ($tag eq "button") {
+                    my $type = delete $attr->{type} || "submit";
+                    $f->push_input($type, $attr);
+                }
 		elsif ($tag eq "textarea") {
 		    $attr->{textarea_value} = $attr->{value}
 		        if exists $attr->{value};
@@ -189,7 +219,7 @@ sub parse
 			    $f->push_input("option", \%a);
 			}
 			else {
-			    Carp::carp("Bad <select> tag '$tag'") if $^W;
+			    warn("Bad <select> tag '$tag' in $base_uri\n") if $verbose;
 			    if ($tag eq "/form" ||
 				$tag eq "input" ||
 				$tag eq "textarea" ||
@@ -214,7 +244,7 @@ sub parse
 	    }
 	}
 	elsif ($form_tags{$tag}) {
-	    Carp::carp("<$tag> outside <form>") if $^W;
+	    warn("<$tag> outside <form> in $base_uri\n") if $verbose;
 	}
     }
     for (@forms) {
@@ -875,7 +905,7 @@ sub form_name_value
     my $self = shift;
     my $name = $self->{'name'};
     return unless defined $name;
-    return if $self->{disabled};
+    return if $self->disabled;
     my $value = $self->value;
     return unless defined $value;
     return ($name => $value);
@@ -936,7 +966,11 @@ sub value
     if (@_) {
         Carp::carp("Input '$self->{name}' is readonly")
 	    if $^W && $self->{readonly};
-	$self->{value} = shift;
+        my $new = shift;
+        my $n = exists $self->{maxlength} ? $self->{maxlength} : undef;
+        Carp::carp("Input '$self->{name}' has maxlength '$n'")
+	    if $^W && defined($n) && defined($new) && length($new) > $n;
+	$self->{value} = $new;
     }
     $old;
 }
@@ -1006,14 +1040,14 @@ sub add_to_form
     return $self->SUPER::add_to_form($form)
 	if $type eq "checkbox";
 
-    if ($type eq "option" && $self->{multiple}) {
-	$self->{disabled} ||= $self->{option_disabled};
+    if ($type eq "option" && exists $self->{multiple}) {
+	$self->{disabled} ||= delete $self->{option_disabled};
 	return $self->SUPER::add_to_form($form);
     }
 
     die "Assert" if @{$self->{menu}} != 1;
     my $m = $self->{menu}[0];
-    $m->{disabled}++ if $self->{option_disabled};
+    $m->{disabled}++ if delete $self->{option_disabled};
 
     my $prev = $form->find_input($self->{name}, $self->{type});
     return $self->SUPER::add_to_form($form) unless $prev;
@@ -1032,6 +1066,29 @@ sub fixup
     $self->{menu}[$self->{current}]{seen}++ if exists $self->{current};
 }
 
+sub disabled
+{
+    my $self = shift;
+    my $type = $self->type;
+
+    my $old = $self->{disabled} || _menu_all_disabled(@{$self->{menu}});
+    if (@_) {
+	my $v = shift;
+	$self->{disabled} = $v;
+        for (@{$self->{menu}}) {
+            $_->{disabled} = $v;
+        }
+    }
+    return $old;
+}
+
+sub _menu_all_disabled {
+    for (@_) {
+	return 0 unless $_->{disabled};
+    }
+    return 1;
+}
+
 sub value
 {
     my $self = shift;
@@ -1327,7 +1384,7 @@ L<LWP>, L<LWP::UserAgent>, L<HTML::Parser>
 
 =head1 COPYRIGHT
 
-Copyright 1998-2003 Gisle Aas.
+Copyright 1998-2005 Gisle Aas.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
