@@ -1,10 +1,8 @@
 package HTTP::Request;
 
-# $Id: Request.pm,v 1.36 2004/04/06 10:44:31 gisle Exp $
-
 require HTTP::Message;
 @ISA = qw(HTTP::Message);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.36 $ =~ /(\d+)\.(\d+)/);
+$VERSION = "5.810";
 
 use strict;
 
@@ -20,6 +18,27 @@ sub new
 }
 
 
+sub parse
+{
+    my($class, $str) = @_;
+    my $request_line;
+    if ($str =~ s/^(.*)\n//) {
+	$request_line = $1;
+    }
+    else {
+	$request_line = $str;
+	$str = "";
+    }
+
+    my $self = $class->SUPER::parse($str);
+    my($method, $uri, $protocol) = split(' ', $request_line);
+    $self->method($method) if defined($method);
+    $self->uri($uri) if defined($uri);
+    $self->protocol($protocol) if $protocol;
+    $self;
+}
+
+
 sub clone
 {
     my $self = shift;
@@ -70,7 +89,8 @@ sub uri
 
 sub as_string
 {
-    my($self, $eol) = @_;
+    my $self = shift;
+    my($eol) = @_;
     $eol = "\n" unless defined $eol;
 
     my $req_line = $self->method || "-";
@@ -80,7 +100,7 @@ sub as_string
     my $proto = $self->protocol;
     $req_line .= " $proto" if $proto;
 
-    return join($eol, $req_line, $self->SUPER::as_string($eol));
+    return join($eol, $req_line, $self->SUPER::as_string(@_));
 }
 
 
@@ -123,10 +143,14 @@ inherits its methods.  The following additional methods are available:
 
 Constructs a new C<HTTP::Request> object describing a request on the
 object $uri using method $method.  The $method argument must be a
-string.  The $uri argument can be either a string, or a reference
-to a C<URI> object.  The optional $header argument should be a
-reference to an C<HTTP::Headers> object.  The optional $content
-argument should be a string of bytes.
+string.  The $uri argument can be either a string, or a reference to a
+C<URI> object.  The optional $header argument should be a reference to
+an C<HTTP::Headers> object or a plain array reference of key/value
+pairs.  The optional $content argument should be a string of bytes.
+
+=item $r = HTTP::Request->parse( $str )
+
+This constructs a new request object by parsing the given string.
 
 =item $r->method
 
@@ -166,8 +190,9 @@ module can be used to turn such strings into a string of bytes.
 
 =item $r->as_string
 
+=item $r->as_string( $eol )
+
 Method returning a textual representation of the request.
-Mainly useful for debugging purposes. It takes no arguments.
 
 =back
 
@@ -178,7 +203,7 @@ L<HTTP::Response>
 
 =head1 COPYRIGHT
 
-Copyright 1995-2001 Gisle Aas.
+Copyright 1995-2004 Gisle Aas.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
