@@ -1,10 +1,8 @@
 package HTTP::Response;
 
-# $Id: Response.pm,v 1.53 2005/12/06 13:19:09 gisle Exp $
-
 require HTTP::Message;
 @ISA = qw(HTTP::Message);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.53 $ =~ /(\d+)\.(\d+)/);
+$VERSION = "5.810";
 
 use strict;
 use HTTP::Status ();
@@ -121,17 +119,18 @@ sub is_error    { HTTP::Status::is_error    (shift->{'_rc'}); }
 
 sub error_as_HTML
 {
+    require HTML::Entities;
     my $self = shift;
     my $title = 'An Error Occurred';
-    my $body  = $self->status_line;
+    my $body  = HTML::Entities::encode($self->status_line);
     return <<EOM;
-<HTML>
-<HEAD><TITLE>$title</TITLE></HEAD>
-<BODY>
-<H1>$title</H1>
-$body
-</BODY>
-</HTML>
+<html>
+<head><title>$title</title></head>
+<body>
+<h1>$title</h1>
+<p>$body</p>
+</body>
+</html>
 EOM
 }
 
@@ -320,7 +319,7 @@ other methods that can be used to access the content.
 =item $r->decoded_content( %options )
 
 This will return the content after any C<Content-Encoding> and
-charsets has been decoded.  See L<HTTP::Message> for details.
+charsets have been decoded.  See L<HTTP::Message> for details.
 
 =item $r->request
 
