@@ -1,10 +1,8 @@
 package LWP::RobotUA;
 
-# $Id: RobotUA.pm,v 1.22 2003/10/23 19:11:32 uid39246 Exp $
-
 require LWP::UserAgent;
 @ISA = qw(LWP::UserAgent);
-$VERSION = sprintf("%d.%02d", q$Revision: 1.22 $ =~ /(\d+)\.(\d+)/);
+$VERSION = "5.810";
 
 require WWW::RobotRules;
 require HTTP::Request;
@@ -28,26 +26,37 @@ use strict;
 
 sub new
 {
-    my($class,$name,$from,$rules) = @_;
+    my $class = shift;
+    my %cnf;
+    if (@_ < 4) {
+	# legacy args
+	@cnf{qw(agent from rules)} = @_;
+    }
+    else {
+	%cnf = @_;
+    }
 
-    Carp::croak('LWP::RobotUA name required') unless $name;
-    Carp::croak('LWP::RobotUA from address required') unless $from
-     and $from =~ m/\@/;
+    Carp::croak('LWP::RobotUA agent required') unless $cnf{agent};
+    Carp::croak('LWP::RobotUA from address required')
+	unless $cnf{from} && $cnf{from} =~ m/\@/;
 
-    my $self = new LWP::UserAgent;
+    my $delay = delete $cnf{delay} || 1;
+    my $use_sleep = delete $cnf{use_sleep};
+    $use_sleep = 1 unless defined($use_sleep);
+    my $rules = delete $cnf{rules};
+
+    my $self = LWP::UserAgent->new(%cnf);
     $self = bless $self, $class;
 
     $self->{'delay'} = 1;   # minutes
-    $self->{'agent'} = $name;
-    $self->{'from'}  = $from;
     $self->{'use_sleep'} = 1;
 
     if ($rules) {
-	$rules->agent($name);
+	$rules->agent($cnf{agent});
 	$self->{'rules'} = $rules;
     }
     else {
-	$self->{'rules'} = new WWW::RobotRules $name;
+	$self->{'rules'} = WWW::RobotRules->new($cnf{agent});
     }
 
     $self;
@@ -126,7 +135,7 @@ sub simple_request
 	my $fresh_until = $robot_res->fresh_until;
 	if ($robot_res->is_success) {
 	    my $c = $robot_res->content;
-	    if ($robot_res->content_type =~ m,^text/, && $c =~ /Disallow/) {
+	    if ($robot_res->content_type =~ m,^text/, && $c =~ /^\s*Disallow\s*:/mi) {
 		LWP::Debug::debug("Parsing robot rules");
 		$self->{'rules'}->parse($robot_url, $c, $fresh_until);
 	    }
@@ -202,7 +211,7 @@ LWP::RobotUA - a class for well-behaved Web robots
 =head1 SYNOPSIS
 
   use LWP::RobotUA;
-  my $ua = LWP::RobotUA->new( 'my-robot/0.1', 'me@foo.com' );
+  my $ua = LWP::RobotUA->new('my-robot/0.1', 'me@foo.com');
   $ua->delay(10);  # be very nice -- max one hit every ten minutes!
   ...
 
@@ -236,43 +245,55 @@ same methods. In addition the following methods are provided:
 
 =over 4
 
-=item $ua = LWP::RobotUA->new($agent_name, $from, [$rules])
+=item $ua = LWP::RobotUA->new( %options )
 
-Your robot's name and the mail address of the human responsible for
-the robot (i.e. you) are required by the constructor.
+=item $ua = LWP::RobotUA->new( $agent, $from )
 
-Optionally it allows you to specify the I<WWW::RobotRules> object to
-use.  If you don't provide one, then this user agent will make its own
-internal database of F<robots.txt> rules as needed.
+=item $ua = LWP::RobotUA->new( $agent, $from, $rules )
 
-=item $ua->delay([$minutes])
+The LWP::UserAgent options C<agent> and C<from> are mandatory.  The
+options C<delay>, C<use_sleep> and C<rules> initialize attributes
+private to the RobotUA.  If C<rules> are not provided, then
+C<WWW::RobotRules> is instantiated providing an internal database of
+F<robots.txt>.
 
-Set the minimum delay between requests to the same server,
-in I<minutes>.  The
-default is 1 minute.  Note that this number doesn't have to be an integer;
-for example, this sets the delay to 10 seconds:
+It is also possible to just pass the value of C<agent>, C<from> and
+optionally C<rules> as plain positional arguments.
+
+=item $ua->delay
+
+=item $ua->delay( $minutes )
+
+Get/set the minimum delay between requests to the same server, in
+I<minutes>.  The default is 1 minute.  Note that this number doesn't
+have to be an integer; for example, this sets the delay to 10 seconds:
 
     $ua->delay(10/60);
 
-=item $ua->use_sleep([$boolean])
+=item $ua->use_sleep
+
+=item $ua->use_sleep( $boolean )
 
 Get/set a value indicating whether the UA should sleep() if requests
-arrive too fast (before $ua->delay minutes has passed).  The default is
-TRUE.  If this value is FALSE then an internal SERVICE_UNAVAILABLE
-response will be generated.  It will have an Retry-After header that
-indicates when it is OK to send another request to this server.
+arrive too fast, defined as $ua->delay minutes not passed since
+last request to the given server.  The default is TRUE.  If this value is
+FALSE then an internal SERVICE_UNAVAILABLE response will be generated.
+It will have an Retry-After header that indicates when it is OK to
+send another request to this server.
 
-=item $ua->rules([$rules])
+=item $ua->rules
+
+=item $ua->rules( $rules )
 
 Set/get which I<WWW::RobotRules> object to use.
 
-=item $ua->no_visits($netloc)
+=item $ua->no_visits( $netloc )
 
-Returns the number of documents fetched from this server host. Yes I
+Returns the number of documents fetched from this server host. Yeah I
 know, this method should probably have been named num_visits() or
 something like that. :-(
 
-=item $ua->host_wait($netloc)
+=item $ua->host_wait( $netloc )
 
 Returns the number of I<seconds> (from now) you must wait before you can
 make a new request to this host.
@@ -290,7 +311,7 @@ L<LWP::UserAgent>, L<WWW::RobotRules>
 
 =head1 COPYRIGHT
 
-Copyright 1996-2002 Gisle Aas.
+Copyright 1996-2004 Gisle Aas.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
